@@ -1,9 +1,6 @@
 /*
  * This file is part of nmealib.
  *
- * Copyright (c) 2008 Timur Sinitsyn
- * Copyright (c) 2011 Ferry Huberts
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -30,17 +27,16 @@
 extern "C" {
 #endif /* __cplusplus */
 
-bool nmea_parse_sentence_has_invalid_chars(const char * str, const size_t str_len, const char * strName, char * report,
-		const size_t reportSize);
+const char * isInvalidNMEACharacter(const char * c);
+const char * nmea_parse_sentence_has_invalid_chars(const char * s, const size_t len);
 
-int nmea_parse_get_sentence_type(const char *s, const int len);
-int nmea_parse_get_sentence_length(const char *s, const int len, int *checksum);
+enum nmeaPACKTYPE nmea_parse_get_sentence_type(const char *s, const int len);
 
-int nmea_parse_GPGGA(const char *s, const int len, nmeaGPGGA *pack);
-int nmea_parse_GPGSA(const char *s, const int len, nmeaGPGSA *pack);
-int nmea_parse_GPGSV(const char *s, const int len, nmeaGPGSV *pack);
-int nmea_parse_GPRMC(const char *s, const int len, nmeaGPRMC *pack);
-int nmea_parse_GPVTG(const char *s, const int len, nmeaGPVTG *pack);
+int nmea_parse_GPGGA(const char *s, const int len, bool has_checksum, nmeaGPGGA *pack);
+int nmea_parse_GPGSA(const char *s, const int len, bool has_checksum, nmeaGPGSA *pack);
+int nmea_parse_GPGSV(const char *s, const int len, bool has_checksum, nmeaGPGSV *pack);
+int nmea_parse_GPRMC(const char *s, const int len, bool has_checksum, nmeaGPRMC *pack);
+int nmea_parse_GPVTG(const char *s, const int len, bool has_checksum, nmeaGPVTG *pack);
 
 #ifdef  __cplusplus
 }
