@@ -1,9 +1,6 @@
 /*
  * This file is part of nmealib.
  *
- * Copyright (c) 2008 Timur Sinitsyn
- * Copyright (c) 2011 Ferry Huberts
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -54,7 +51,7 @@ void nmea_time_now(nmeaTIME *utc, uint32_t * present) {
 	utc->sec = tt.tm_sec;
 	utc->hsec = (tp.tv_usec / 10000);
 	if (present) {
-		*present |= (UTCDATE | UTCTIME);
+	  nmea_INFO_set_present(present, UTCDATE | UTCTIME);
 	}
 }
 
