@@ -53,6 +53,12 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#ifdef __linux__
+#include <linux/rtnetlink.h>
+#else
+#define RTPROT_BOOT 3
+#endif
+
 int olsrd_write_cnf(struct olsrd_config *cnf, const char *fname) {
   FILE *fd;
   struct autobuf abuf;
@@ -103,7 +109,36 @@ if_appendf(struct autobuf *autobuf, bool comments, const char *fmt, ...)
   return rv;
 }
 
-static void olsrd_write_if_autobuf(struct autobuf *out, struct if_config_options *cnfi, bool comments) {
+static void olsrd_write_hna_autobuf(int ip_version, struct ip_prefix_list *hna, struct autobuf *out, struct olsrd_config *cnf) {
+  struct ip_prefix_list *hna1 = (cnf->ip_version == ip_version) ? hna : NULL;
+
+  abuf_appendf(out, "Hna%u\n"
+    "{\n",
+    ip_version == AF_INET ? 4 : 6);
+
+  if (!hna1) {
+    if (ip_version == AF_INET)
+      abuf_puts(out,
+          "# Internet gateway\n"
+          "# 0.0.0.0   0.0.0.0\n"
+          "# specific small networks reachable through this node\n"
+          "# 15.15.0.0 255.255.255.0\n");
+    else
+      abuf_puts(out,
+          "# Internet gateway\n"
+          "#   0::                     0\n"
+          "# specific small networks reachable through this node\n"
+          "#   fec0:2200:106:0:0:0:0:0 48\n");
+  } else
+    while (hna1) {
+      struct ipaddr_str strbuf;
+      abuf_appendf(out, "    %s\n", olsr_ip_prefix_to_string(&hna1->net));
+      hna1 = hna1->next;
+    }
+  abuf_appendf(out, "}\n");
+}
+
+static void olsrd_write_if_autobuf(struct autobuf *out, struct olsrd_config *cnf, struct if_config_options *cnfi, bool comments) {
   struct ipaddr_str ipbuf;
   struct olsr_lq_mult *mult;
 
@@ -113,12 +148,12 @@ static void olsrd_write_if_autobuf(struct autobuf *out, struct if_config_options
     "    # packet forwarding on switched ethernet interfaces\n"
     "    # valid Modes are \"mesh\" and \"ether\"\n"
     "    # (default is \"%s\")\n"
-    "    \n", OLSR_IF_MODE[DEF_IF_MODE]);
+    "\n", OLSR_IF_MODE[DEF_IF_MODE]);
   if_appendf(out, comments, "    %sMode \"%s\"\n",
       cnfi->mode == DEF_IF_MODE ? "# " : "",
       OLSR_IF_MODE[cnfi->mode]);
   if (comments) abuf_puts(out,
-    "    \n"
+    "\n"
     "    # IPv4 broadcast address for outgoing OLSR packets.\n"
     "    # One useful example would be 255.255.255.255\n"
     "    # The second useful value would be to\n"
@@ -126,43 +161,48 @@ static void olsrd_write_if_autobuf(struct autobuf *out, struct if_config_options
     "    # another name of this parameter is \"IPv4Multicast\"\n"
     "    # (default is 0.0.0.0, which triggers the usage of the\n"
     "    # interface broadcast IP)\n"
-    "    \n");
+    "\n");
   if_appendf(out, comments, "    %sIp4Broadcast      %s\n",
       cnfi->ipv4_multicast.v4.s_addr == 0 ? "# " : "",
       inet_ntop(AF_INET, &cnfi->ipv4_multicast, ipbuf.buf, sizeof(ipbuf)));
   if (comments) abuf_appendf(out,
-    "    \n"
+    "\n"
     "    # IPv6 multicast address\n"
     "    # (default is %s, the manet-router linklocal multicast)\n"
-    "    \n", inet_ntop(AF_INET6, &ipv6_def_multicast, ipbuf.buf, sizeof(ipbuf)));
+    "\n", inet_ntop(AF_INET6, &ipv6_def_multicast, ipbuf.buf, sizeof(ipbuf)));
   if_appendf(out, comments, "    %sIPv6Multicast %s\n",
       memcmp(&cnfi->ipv6_multicast, &ipv6_def_multicast, sizeof(ipv6_def_multicast)) == 0 ? "# " : "",
       inet_ntop(AF_INET6, &cnfi->ipv6_multicast, ipbuf.buf, sizeof(ipbuf)));
   if (comments) abuf_puts(out,
-    "    \n"
+    "\n"
     "    # IPv4 src address for outgoing OLSR packages\n"
     "    # (default is 0.0.0.0, which triggers usage of the interface IP)\n"
-    "    \n");
+    "\n");
   if_appendf(out, comments, "    %sIPv4Src %s\n",
       cnfi->ipv4_src.v4.s_addr == 0 ? "# " : "",
       inet_ntop(AF_INET, &cnfi->ipv4_src, ipbuf.buf, sizeof(ipbuf)));
   if (comments) abuf_puts(out,
-    "    \n"
+    "\n"
     "    # IPv6 src prefix. OLSRd will choose one of the interface IPs\n"
     "    # which matches the prefix of this parameter.\n"
     "    # (default is 0::/0, which triggers the usage\n"
     "    # of a not-linklocal interface IP)\n"
-    "    \n");
-  if_appendf(out, comments, "    %sIPv6Src %s\n",
+    "\n");
+  {
+    int saved = cnf->ip_version;
+    cnf->ip_version = AF_INET6;
+    if_appendf(out, comments, "    %sIPv6Src %s\n",
       cnfi->ipv6_src.prefix_len == 0 ? "# " : "",
-      inet_ntop(AF_INET6, &cnfi->ipv6_src, ipbuf.buf, sizeof(ipbuf)));
+          olsr_ip_prefix_to_string(&cnfi->ipv6_src));
+    cnf->ip_version = saved;
+  }
   if (comments) abuf_appendf(out,
-    "    \n"
+    "\n"
     "    # Emission intervals in seconds.\n"
     "    # If not defined, Freifunk network defaults are used.\n"
     "    # (defaults: Hello = %.1f/%.1f,  TC  = %.1f/%.1f,\n"
     "    #            MID   = %.1f/%.1f, HNA = %.1f/%.1f)\n"
-    "    \n",
+    "\n",
     (double)HELLO_INTERVAL, (double)NEIGHB_HOLD_TIME,
     (double)TC_INTERVAL,    (double)TOP_HOLD_TIME,
     (double)MID_INTERVAL,   (double)MID_HOLD_TIME,
@@ -193,7 +233,7 @@ static void olsrd_write_if_autobuf(struct autobuf *out, struct if_config_options
       cnfi->hna_params.validity_time == HNA_HOLD_TIME ? "# " : "",
       (double)cnfi->hna_params.validity_time);
   if (comments) abuf_puts(out,
-    "    \n"
+    "\n"
     "    # When multiple links exist between hosts\n"
     "    # the weight of interface is used to determine\n"
     "    # the link to use. Normally the weight is\n"
@@ -209,7 +249,7 @@ static void olsrd_write_if_autobuf(struct autobuf *out, struct if_config_options
       !cnfi->weight.fixed ? "# " : "",
       cnfi->weight.value);
   if (comments) abuf_puts(out,
-    "    \n"
+    "\n"
     "    # If a certain route should be preferred\n"
     "    # or ignored by the mesh, the Link Quality\n"
     "    # value of a node can be multiplied with a factor\n"
@@ -221,11 +261,17 @@ static void olsrd_write_if_autobuf(struct autobuf *out, struct if_config_options
     "    # Note:\n"
     "    # Link quality multiplier is used only when\n"
     "    # LinkQualityLevel is greater than 0.\n"
-    "    \n");
+    "\n");
   mult = cnfi->lq_mult;
 
   if (mult == NULL) {
-    if (comments) abuf_puts(out, "    # LinkQualityMult 192.168.0.1 0.5\n");
+    if (comments) {
+      abuf_puts(out, "    # example 1: reduce LQ to 192.168.0.1 by half\n");
+      abuf_puts(out, "    # LinkQualityMult 192.168.0.1 0.5\n");
+      abuf_puts(out, "\n");
+      abuf_puts(out, "    # example 2: reduce LQ to all nodes on this interface by 20%\n");
+      abuf_puts(out, "    # LinkQualityMult default 0.8\n");
+    }
   } else {
     while (mult != NULL) {
       if_appendf(out, comments, "    LinkQualityMult    %s %0.2f\n",
@@ -314,14 +360,18 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# The lockfile is used to prevent multiple OLSR instances running at the same\n"
     "# time.\n"
     "# (Linux/BSD default is \"/var/run/olsrd-ipv(4/6).lock\")\n"
-    "# (Win32     default is \"configfile-ipv(4/6).lock\")\n"
+    "# (Win32     default is \"[configfile]-ipv(4/6).lock\")\n"
     "\n");
-  abuf_appendf(out, "%sLockFile \"%s\"\n",
-      cnf->lock_file == NULL ? "# " : "",
-      cnf->lock_file ? cnf->lock_file : "lockfile");
+  {
+    char * lockfile_default = olsrd_get_default_lockfile(cnf);
+    abuf_appendf(out, "%sLockFile \"%s\"\n",
+        !strcmp(cnf->lock_file, lockfile_default) ? "# " : "",
+        cnf->lock_file);
+    free(lockfile_default);
+  }
   abuf_appendf(out,
     "\n"
-    "# Polling rate for OLSR sockets in seconds (float). \n"
+    "# Polling rate for OLSR sockets in seconds (float).\n"
     "# (default is %.2f)\n"
     "\n", (double)DEF_POLLRATE);
   abuf_appendf(out, "%sPollrate  %.2f\n",
@@ -347,9 +397,9 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
   abuf_appendf(out,
     "\n"
     "# FIBMetric controls the metric value of the host-routes OLSRd sets.\n"
-    "# - \"flat\" means that the metric value is always 2(or as configured \n"
-    "#   with FIBMetricDefault). This is the preferred value because it \n"
-    "#   helps the linux kernel routing to clean up older routes\n"
+    "# - \"flat\" means that the metric value is always 2 (or as configured\n"
+    "#   with FIBMetricDefault). This is the preferred value because it helps\n"
+    "#   the linux kernel routing to clean up older routes\n"
     "# - \"correct\" use the hopcount as the metric value.\n"
     "# - \"approx\" use the hopcount as the metric value too, but does only update the\n"
     "#   hopcount if the nexthop changes too\n"
@@ -392,14 +442,20 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# 1 gets remapped by olsrd to 0 UNSPECIFIED (1 is reserved for ICMP redirects)\n"
     "# 2 KERNEL routes (not very wise to use)\n"
     "# 3 BOOT (should in fact not be used by routing daemons)\n"
-    "# 4 STATIC \n"
+    "# 4 STATIC\n"
     "# 8 .. 15 various routing daemons (gated, zebra, bird, & co)\n"
-    "# (default is %u which gets replaced by an OS-specific default value\n"
-    "# under linux 3 (BOOT) (for backward compatibility)\n"
-    "\n", DEF_RTPROTO);
-  abuf_appendf(out, "%sRtProto %u\n",
-      cnf->rt_proto == DEF_RTPROTO ? "# " : "",
-      cnf->rt_proto);
+    "# (default is %u which gets replaced by an OS-specific default value;\n"
+    "# %u (BOOT) under linux (for backward compatibility)\n"
+    "\n", DEF_RTPROTO, RTPROT_BOOT);
+  {
+    int expected = DEF_RTPROTO;
+#ifdef __linux__
+    expected = RTPROT_BOOT;
+#endif
+    abuf_appendf(out, "%sRtProto %u\n",
+        cnf->rt_proto == expected ? "# " : "",
+        cnf->rt_proto == expected ? DEF_RTPROTO : cnf->rt_proto);
+  }
   abuf_appendf(out,
     "\n"
     "# Specifies the routing Table olsr uses\n"
@@ -413,25 +469,55 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     DEF_SGW_RT_TABLE_NR, DEF_SGW_RT_TABLE_DEFAULT_NR, DEF_SGW_RT_TABLE_TUNNEL_NR,
     DEF_RT_TABLE_NR, DEF_RT_TABLE_DEFAULT_NR, DEF_RT_TABLE_TUNNEL_NR);
   if (!cnf->smart_gw_active) {
-    abuf_appendf(out, "%sRtTable        %u\n",
-        cnf->rt_table == DEF_RT_TABLE_NR ? "# " : "",
-        cnf->rt_table);
-    abuf_appendf(out, "%sRtTableDefault %u\n",
-        cnf->rt_table_default == DEF_RT_TABLE_DEFAULT_NR ? "# " : "",
-        cnf->rt_table_default);
-    abuf_appendf(out, "%sRtTableTunnel  %u\n",
-        cnf->rt_table_tunnel == DEF_RT_TABLE_TUNNEL_NR ? "# " : "",
-        cnf->rt_table_tunnel);
+    if (cnf->rt_table == DEF_RT_TABLE_NR)
+      abuf_appendf(out, "%sRtTable        %s\n",
+          "# ",
+          "auto");
+    else
+      abuf_appendf(out, "%sRtTable        %u\n",
+          "",
+          cnf->rt_table);
+    if (cnf->rt_table_default == DEF_RT_TABLE_DEFAULT_NR)
+      abuf_appendf(out, "%sRtTableDefault %s\n",
+          "# ",
+          "auto");
+    else
+      abuf_appendf(out, "%sRtTableDefault %u\n",
+          "",
+          cnf->rt_table_default);
+    if (cnf->rt_table_tunnel == DEF_RT_TABLE_TUNNEL_NR)
+      abuf_appendf(out, "%sRtTableTunnel  %s\n",
+          "# ",
+          "auto");
+    else
+      abuf_appendf(out, "%sRtTableTunnel  %u\n",
+          "",
+          cnf->rt_table_tunnel);
   } else {
-    abuf_appendf(out, "%sRtTable        %u\n",
-        cnf->rt_table == DEF_SGW_RT_TABLE_NR ? "# " : "",
-        cnf->rt_table);
-    abuf_appendf(out, "%sRtTableDefault %u\n",
-        cnf->rt_table_default == DEF_SGW_RT_TABLE_DEFAULT_NR ? "# " : "",
-        cnf->rt_table_default);
-    abuf_appendf(out, "%sRtTableTunnel  %u\n",
-        cnf->rt_table_tunnel == DEF_SGW_RT_TABLE_TUNNEL_NR ? "# " : "",
-        cnf->rt_table_tunnel);
+    if (cnf->rt_table == DEF_SGW_RT_TABLE_NR)
+      abuf_appendf(out, "%sRtTable        %s\n",
+          "# ",
+          "auto");
+    else
+      abuf_appendf(out, "%sRtTable        %u\n",
+          "",
+          cnf->rt_table);
+    if (cnf->rt_table_default == DEF_SGW_RT_TABLE_DEFAULT_NR)
+      abuf_appendf(out, "%sRtTableDefault %s\n",
+          "# ",
+          "auto");
+    else
+      abuf_appendf(out, "%sRtTableDefault %u\n",
+          "",
+          cnf->rt_table_default);
+    if (cnf->rt_table_tunnel == DEF_SGW_RT_TABLE_TUNNEL_NR)
+      abuf_appendf(out, "%sRtTableTunnel  %s\n",
+          "# ",
+          "auto");
+    else
+      abuf_appendf(out, "%sRtTableTunnel  %u\n",
+          "",
+          cnf->rt_table_tunnel);
   }
   abuf_appendf(out,
     "\n"
@@ -442,74 +528,120 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# RtTablePriority less than RtTableDefaultOlsrPriority\n"
     "# less than RtTableTunnelPriority less than RtTableDefaultPriority\n"
     "# There are two special parameters, \"auto\" (choose fitting to SmartGW\n"
-    "# mode) and \"none\" (do not set policy rule)\n"
-    "# (with    smartgw: default is %d/%u/%u/%u)\n"
-    "# (without smartgw: default is %d/%d   /%d   /%d   )\n"
-    "\n",
-    DEF_SGW_RT_TABLE_PRI,
+    "# mode) and \"none\" (do not set policy rule)\n");
+
+  abuf_puts(out, "# (with    smartgw: default is ");
+  if (cnf->rt_table_pri > 0)
+    abuf_appendf(out,
+      "%5d/",
+      DEF_SGW_RT_TABLE_PRI);
+  else
+    abuf_appendf(out,
+      "%5s/",
+      "none");
+  abuf_appendf(out,
+    "%-5u/%-5u/%-5u)\n",
     DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER,
     DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER + DEF_SGW_RT_TABLE_TUNNEL_PRI_ADDER,
-    DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER + DEF_SGW_RT_TABLE_TUNNEL_PRI_ADDER + DEF_SGW_RT_TABLE_DEFAULT_PRI_ADDER,
-    DEF_RT_TABLE_PRI,
-    DEF_RT_TABLE_DEFAULTOLSR_PRI,
-    DEF_RT_TABLE_TUNNEL_PRI,
-    DEF_RT_TABLE_DEFAULT_PRI);
+    DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER + DEF_SGW_RT_TABLE_TUNNEL_PRI_ADDER + DEF_SGW_RT_TABLE_DEFAULT_PRI_ADDER);
+
+  abuf_puts(out, "# (without smartgw: default is ");
+  if (cnf->rt_table_pri == DEF_RT_AUTO || cnf->rt_table_pri == DEF_RT_TABLE_PRI)
+    abuf_appendf(out,
+      "%5s/",
+      "auto");
+  else
+    abuf_appendf(out,
+      "%5d/",
+      DEF_RT_TABLE_PRI);
+  if (cnf->rt_table_defaultolsr_pri == DEF_RT_AUTO || cnf->rt_table_defaultolsr_pri == DEF_RT_TABLE_DEFAULTOLSR_PRI)
+    abuf_appendf(out,
+      "%-5s/",
+      "auto");
+  else
+    abuf_appendf(out,
+      "%-5d/",
+      DEF_RT_TABLE_DEFAULTOLSR_PRI);
+  if (cnf->rt_table_tunnel_pri == DEF_RT_AUTO || cnf->rt_table_tunnel_pri == DEF_RT_TABLE_TUNNEL_PRI)
+    abuf_appendf(out,
+      "%-5s/",
+      "auto");
+  else
+    abuf_appendf(out,
+      "%-5d/",
+      DEF_RT_TABLE_TUNNEL_PRI);
+  if (cnf->rt_table_default_pri == DEF_RT_AUTO || cnf->rt_table_default_pri == DEF_RT_TABLE_DEFAULT_PRI)
+    abuf_appendf(out,
+      "%-5s",
+      "auto");
+  else
+    abuf_appendf(out,
+      "%-5d",
+      DEF_RT_TABLE_DEFAULT_PRI);
+  abuf_puts(out, ")\n");
+
+  abuf_puts(out, "\n");
+
   if (!cnf->smart_gw_active) {
-    if (cnf->rt_table_pri == DEF_RT_TABLE_PRI) {
-      abuf_appendf(out, "# RtTablePriority            %d\n",
+    if (cnf->rt_table_pri == DEF_RT_AUTO || cnf->rt_table_pri == DEF_RT_TABLE_PRI)
+      abuf_appendf(out, "# RtTablePriority            %s\n",
+          "auto");
+    else
+      abuf_appendf(out, "RtTablePriority              %u\n",
           cnf->rt_table_pri);
-    } else {
-      abuf_appendf(out, "RtTablePriority            %u\n",
-          cnf->rt_table_pri);
-    }
-    if (cnf->rt_table_defaultolsr_pri == DEF_RT_TABLE_DEFAULTOLSR_PRI) {
-      abuf_appendf(out, "# RtTableDefaultOlsrPriority %d\n",
+
+    if (cnf->rt_table_defaultolsr_pri == DEF_RT_AUTO || cnf->rt_table_defaultolsr_pri == DEF_RT_TABLE_DEFAULTOLSR_PRI)
+      abuf_appendf(out, "# RtTableDefaultOlsrPriority %s\n",
+          "auto");
+    else
+      abuf_appendf(out, "RtTableDefaultOlsrPriority   %u\n",
           cnf->rt_table_defaultolsr_pri);
-    } else {
-      abuf_appendf(out, "RtTableDefaultOlsrPriority %u\n",
-          cnf->rt_table_defaultolsr_pri);
-    }
-    if (cnf->rt_table_tunnel_pri == DEF_RT_TABLE_TUNNEL_PRI) {
-      abuf_appendf(out, "# RtTableTunnelPriority      %d\n",
+
+    if (cnf->rt_table_tunnel_pri == DEF_RT_AUTO || cnf->rt_table_tunnel_pri == DEF_RT_TABLE_TUNNEL_PRI)
+      abuf_appendf(out, "# RtTableTunnelPriority      %s\n",
+          "auto");
+    else
+      abuf_appendf(out, "RtTableTunnelPriority        %u\n",
           cnf->rt_table_tunnel_pri);
-    } else {
-      abuf_appendf(out, "RtTableTunnelPriority      %u\n",
-          cnf->rt_table_tunnel_pri);
-    }
-    if (cnf->rt_table_default_pri == DEF_RT_TABLE_DEFAULT_PRI) {
-      abuf_appendf(out, "# RtTableDefaultPriority     %d\n",
+
+    if (cnf->rt_table_default_pri == DEF_RT_AUTO || cnf->rt_table_default_pri == DEF_RT_TABLE_DEFAULT_PRI)
+      abuf_appendf(out, "# RtTableDefaultPriority     %s\n",
+          "auto");
+    else
+      abuf_appendf(out, "RtTableDefaultPriority       %u\n",
           cnf->rt_table_default_pri);
-    } else {
-      abuf_appendf(out, "RtTableDefaultPriority     %u\n",
-          cnf->rt_table_default_pri);
-    }
   } else {
-    if (cnf->rt_table_pri == DEF_SGW_RT_TABLE_PRI) {
-      abuf_appendf(out, "# RtTablePriority            %d\n",
+    if (cnf->rt_table_pri > 0)
+      abuf_appendf(out, "RtTablePriority              %u\n",
           cnf->rt_table_pri);
-    } else {
-      abuf_appendf(out, "RtTablePriority            %u\n",
-          cnf->rt_table_pri);
-    }
+    else
+      abuf_appendf(out, "# RtTablePriority            %s\n",
+          "none");
+
     abuf_appendf(out, "%sRtTableDefaultOlsrPriority %u\n",
         cnf->rt_table_defaultolsr_pri == (DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER) ? "# " : "",
         cnf->rt_table_defaultolsr_pri);
+
     abuf_appendf(out, "%sRtTableTunnelPriority      %u\n",
         cnf->rt_table_tunnel_pri == (DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER + DEF_SGW_RT_TABLE_TUNNEL_PRI_ADDER) ? "# " : "",
         cnf->rt_table_tunnel_pri);
+
     abuf_appendf(out, "%sRtTableDefaultPriority     %u\n",
         cnf->rt_table_default_pri == (DEF_SGW_RT_TABLE_PRI_BASE + DEF_SGW_RT_TABLE_DEFAULTOLSR_PRI_ADDER + DEF_SGW_RT_TABLE_TUNNEL_PRI_ADDER + DEF_SGW_RT_TABLE_DEFAULT_PRI_ADDER) ? "# " : "",
         cnf->rt_table_default_pri);
   }
-  abuf_appendf(out,
-    "\n"
-    "# Activates (in IPv6 mode) the automatic use of NIIT\n"
-    "# (see README-Olsr-Extensions)\n"
-    "# (default is \"%s\")\n"
-    "\n", DEF_USE_NIIT ? "yes" : "no");
-  abuf_appendf(out, "%sUseNiit %s\n",
-      cnf->use_niit == DEF_USE_NIIT ? "# " : "",
-      cnf->use_niit ? "yes" : "no");
+  {
+    bool expected = cnf->ip_version == AF_INET ? false : DEF_USE_NIIT;
+    abuf_appendf(out,
+      "\n"
+      "# Activates (in IPv6 mode) the automatic use of NIIT\n"
+      "# (see README-Olsr-Extensions)\n"
+      "# (default is \"%s\" in IPv4 mode, \"%s\" in IPv6 mode)\n"
+      "\n", expected ? "yes" : "no", DEF_USE_NIIT ? "yes" : "no");
+    abuf_appendf(out, "%sUseNiit %s\n",
+        cnf->use_niit == expected ? "# " : "",
+        cnf->use_niit ? "yes" : "no");
+  }
   abuf_appendf(out,
     "\n"
     "# Activates the smartgateway ipip tunnel feature.\n"
@@ -561,7 +693,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "\n");
   abuf_appendf(out, "%sSmartGatewayPolicyRoutingScript %s\n",
       !cnf->smart_gw_policyrouting_script ? "# " : "",
-      !cnf->smart_gw_policyrouting_script ? "" : cnf->smart_gw_policyrouting_script);
+      !cnf->smart_gw_policyrouting_script ? "\"\"" : cnf->smart_gw_policyrouting_script);
   abuf_puts(out,
     "\n"
     "# Determines the egress interfaces that are part of the multi-gateway setup and\n"
@@ -573,10 +705,13 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
       !cnf->smart_gw_egress_interfaces ? "# " : "");
   {
     struct sgw_egress_if * sgwegressif = olsr_cnf->smart_gw_egress_interfaces;
-    while (sgwegressif) {
-      abuf_appendf(out, " \"%s\"", sgwegressif->name);
-      sgwegressif = sgwegressif->next;
-    }
+    if (!sgwegressif)
+      abuf_puts(out, " \"\"");
+    else
+      while (sgwegressif) {
+        abuf_appendf(out, " \"%s\"", sgwegressif->name);
+        sgwegressif = sgwegressif->next;
+      }
     abuf_puts(out, "\n");
   }
   abuf_appendf(out,
@@ -602,13 +737,13 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "\n", DEF_GW_EGRESS_FILE);
   {
     bool def = !cnf->smart_gw_egress_file || !strcmp(cnf->smart_gw_egress_file, DEF_GW_EGRESS_FILE);
-    abuf_appendf(out, "%sSmartGatewayEgressFile %s\n",
+    abuf_appendf(out, "%sSmartGatewayEgressFile \"%s\"\n",
       def ? "# " : "", def ? DEF_GW_EGRESS_FILE : cnf->smart_gw_egress_file);
   }
   abuf_appendf(out,
     "\n"
-    "# SmartGatewayEgressFilePeriod determines the period (in milliseconds) on which\n"
-    "# the SmartGatewayEgressFile is checked for changes and processed if changed.\n"
+    "# Determines the period (in milliseconds) on which the SmartGatewayEgressFile\n"
+    "# is checked for changes and processed if changed.\n"
     "# (default is %u)\n"
     "\n", DEF_GW_EGRESS_FILE_PERIOD);
   abuf_appendf(out, "%sSmartGatewayEgressFilePeriod %u\n",
@@ -678,7 +813,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# gateway is switched to the other gateway. The unit is percentage.\n"
     "# (default is %u)\n"
     "\n", DEF_GW_THRESH);
-  abuf_appendf(out, "%sSmartGatewayThreshold  %d\n",
+  abuf_appendf(out, "%sSmartGatewayThreshold %d\n",
       cnf->smart_gw_thresh == DEF_GW_THRESH ? "# " : "",
       cnf->smart_gw_thresh);
   abuf_appendf(out,
@@ -687,7 +822,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# See README-Olsr-Extensions for a description of smart gateways.\n"
     "# (default is %lu)\n"
     "\n", (long unsigned int)DEF_GW_WEIGHT_EXITLINK_UP);
-  abuf_appendf(out, "%sSmartGatewayWeightExitLinkUp  %d\n",
+  abuf_appendf(out, "%sSmartGatewayWeightExitLinkUp %d\n",
       cnf->smart_gw_weight_exitlink_up == DEF_GW_WEIGHT_EXITLINK_UP ? "# " : "",
       cnf->smart_gw_weight_exitlink_up);
   abuf_appendf(out,
@@ -696,7 +831,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# See README-Olsr-Extensions for a description of smart gateways.\n"
     "# (default is %lu)\n"
     "\n", (long unsigned int)DEF_GW_WEIGHT_EXITLINK_DOWN);
-  abuf_appendf(out, "%sSmartGatewayWeightExitLinkDown  %d\n",
+  abuf_appendf(out, "%sSmartGatewayWeightExitLinkDown %d\n",
       cnf->smart_gw_weight_exitlink_down == DEF_GW_WEIGHT_EXITLINK_DOWN ? "# " : "",
       cnf->smart_gw_weight_exitlink_down);
   abuf_appendf(out,
@@ -705,7 +840,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# See README-Olsr-Extensions for a description of smart gateways.\n"
     "# (default is %u)\n"
     "\n", DEF_GW_WEIGHT_ETX);
-  abuf_appendf(out, "%sSmartGatewayWeightEtx  %d\n",
+  abuf_appendf(out, "%sSmartGatewayWeightEtx %d\n",
       cnf->smart_gw_weight_etx == DEF_GW_WEIGHT_ETX ? "# " : "",
       cnf->smart_gw_weight_etx);
   abuf_appendf(out,
@@ -714,7 +849,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# See README-Olsr-Extensions for a description of smart gateways.\n"
     "# (default is %u)\n"
     "\n", DEF_GW_DIVIDER_ETX);
-  abuf_appendf(out, "%sSmartGatewayDividerEtx  %d\n",
+  abuf_appendf(out, "%sSmartGatewayDividerEtx %d\n",
       cnf->smart_gw_divider_etx == DEF_GW_DIVIDER_ETX ? "# " : "",
       cnf->smart_gw_divider_etx);
   abuf_appendf(out,
@@ -749,11 +884,17 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "\n"
     "# Specifies the EXTERNAL ipv6 prefix of the uplink. A prefix\n"
     "# length of more than 64 is not allowed.\n"
-    "# (default is 0::/0\n"
+    "# Only relevant when running in IPv6 mode.\n"
+    "# (default is 0::/0)\n"
     "\n");
-  abuf_appendf(out, "%sSmartGatewayPrefix %s\n",
-      cnf->smart_gw_prefix.prefix_len == 0 ? "# " : "",
-      olsr_ip_prefix_to_string(&cnf->smart_gw_prefix));
+  {
+    int saved = cnf->ip_version;
+    cnf->ip_version = AF_INET6;
+    abuf_appendf(out, "%sSmartGatewayPrefix %s\n",
+        cnf->smart_gw_prefix.prefix_len == 0 ? "# " : "",
+        olsr_ip_prefix_to_string(&cnf->smart_gw_prefix));
+    cnf->ip_version = saved;
+  }
   abuf_appendf(out,
     "\n"
     "##############################\n"
@@ -761,7 +902,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "##############################\n"
     "\n"
     "# For testing purposes it may be nice to use another port for olsrd\n"
-    "# for using another port than the IANA assigned one \n"
+    "# for using another port than the IANA assigned one\n"
     "# for a production network, there should be a good reason!!\n"
     "# valid values are integers greater than 1, please be careful with\n"
     "# using reserved port numbers\n"
@@ -776,8 +917,14 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# change during the uptime of olsrd.\n"
     "# (default is 0.0.0.0, which triggers usage of the IP of the first interface)\n"
     "\n");
-  abuf_appendf(out, "MainIp %s\n",
-      olsr_ip_to_string(&ipbuf, &cnf->main_addr));
+  {
+    union olsr_ip_addr main_addr;
+    memset(&main_addr, 0, sizeof(main_addr));
+
+    abuf_appendf(out, "%sMainIp %s\n",
+        !memcmp(&main_addr, &cnf->main_addr, (cnf->ip_version == AF_INET) ? sizeof(main_addr.v4) : sizeof(main_addr.v6)) ? "# " : "",
+        olsr_ip_to_string(&ipbuf, &cnf->main_addr));
+  }
   abuf_appendf(out,
     "\n"
     "# The fixed willingness to use (0-7)\n"
@@ -796,16 +943,9 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# Syntax for HNA4 is \"network-address    network-mask\"\n"
     "# Syntax for HNA6 is \"network-address    prefix-length\"\n"
     "# (default is no HNA)\n");
-  abuf_appendf(out, "Hna%u\n"
-    "{\n",
-    cnf->ip_version == AF_INET ? 4 : 6);
-  while (hna) {
-    struct ipaddr_str strbuf;
-    abuf_appendf(out, "    %s\n", olsr_ip_prefix_to_string(&hna->net));
-    hna = hna->next;
-  }
+  olsrd_write_hna_autobuf(AF_INET, hna, out, cnf);
+  olsrd_write_hna_autobuf(AF_INET6, hna, out, cnf);
   abuf_appendf(out,
-    "}\n"
     "\n"
     "# Hysteresis for link sensing (only for hopcount metric)\n"
     "# Hysteresis adds more robustness to the link sensing\n"
@@ -830,7 +970,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
   abuf_appendf(out, "%sHystThrHigh  %.2f\n",
       cnf->hysteresis_param.thr_high == (float)HYST_THRESHOLD_HIGH ? "# " : "",
       (double)cnf->hysteresis_param.thr_high);
-  abuf_appendf(out, "%sHystThrLow  %.2f\n",
+  abuf_appendf(out, "%sHystThrLow   %.2f\n",
       cnf->hysteresis_param.thr_low == (float)HYST_THRESHOLD_LOW ? "# " : "",
       (double)cnf->hysteresis_param.thr_low);
   abuf_appendf(out,
@@ -862,7 +1002,7 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "### OLSR protocol extensions ###\n"
     "################################\n"
     "\n"
-    "# Link quality level switch between hopcount and \n"
+    "# Link quality level switch between hopcount and\n"
     "# cost-based (mostly ETX) routing. Because of\n"
     "# a design problem in the 0.5.x dijkstra algorithm this\n"
     "# value should not be set to 1.\n"
@@ -910,11 +1050,11 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
   abuf_appendf(out,
     "\n"
     "#\n"
-    "# NatThreshold \n"
+    "# NatThreshold\n"
     "#\n"
     "# (currently this is only in the freifunk firmware)\n"
     "# If the NAT-Endpoint (the preferred 0/0 HNA emitting node)\n"
-    "# is to be changed, the ETX value of the current 0/0 is \n"
+    "# is to be changed, the ETX value of the current 0/0 is\n"
     "# multiplied with the NatThreshold value before being\n"
     "# compared to the new one.\n"
     "# The parameter can be a value between 0.1 and 1.0, but\n"
@@ -935,28 +1075,33 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "\n"
     "IpcConnect\n"
     "{\n"
-    "    # Determines how many simultaneously\n"
-    "    # IPC connections that will be allowed\n"
-    "    # Setting this to 0 disables IPC\n"
+    "     # Determines how many simultaneously\n"
+    "     # IPC connections that will be allowed\n"
+    "     # Setting this to 0 disables IPC\n"
     "\n");
-  abuf_appendf(out, "  %sMaxConnections  %d\n",
+  abuf_appendf(out, "     %sMaxConnections  %d\n",
       cnf->ipc_connections == DEF_IPC_CONNECTIONS ? "# " : "",
       cnf->ipc_connections);
   abuf_puts(out,
     "\n"
-    "    # By default only 127.0.0.1 is allowed\n"
-    "    # to connect. Here allowed hosts and networks can\n"
-    "    # be added\n"
+    "     # By default only 127.0.0.1 is allowed\n"
+    "     # to connect. Here allowed hosts and networks can\n"
+    "     # be added\n"
     "\n");
-
-  while (ipc_nets) {
-    if (ipc_nets->net.prefix_len == olsr_cnf->maxplen) {
-      abuf_appendf(out, "    Host %s\n", olsr_ip_to_string(&ipbuf, &ipc_nets->net.prefix));
-    } else {
-      abuf_appendf(out, "    Net  %s\n", olsr_ip_prefix_to_string(&ipc_nets->net));
+  if (!ipc_nets)
+    abuf_puts(out,
+      "     # Host            127.0.0.1\n"
+      "     # Host            10.0.0.5\n"
+      "     # Net             192.168.1.0 255.255.255.0\n");
+  else
+    while (ipc_nets) {
+      if (ipc_nets->net.prefix_len == olsr_cnf->maxplen) {
+        abuf_appendf(out, "    Host %s\n", olsr_ip_to_string(&ipbuf, &ipc_nets->net.prefix));
+      } else {
+        abuf_appendf(out, "    Net  %s\n", olsr_ip_prefix_to_string(&ipc_nets->net));
+      }
+      ipc_nets = ipc_nets->next;
     }
-    ipc_nets = ipc_nets->next;
-  }
   abuf_puts(out,
     "}\n"
     "\n"
@@ -966,17 +1111,21 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# Olsrd plugins to load\n"
     "# This must be the absolute path to the file\n"
     "# or the loader will use the following scheme:\n"
-    "# - Try the paths in the LD_LIBRARY_PATH \n"
+    "# - Try the paths in the LD_LIBRARY_PATH\n"
     "#   environment variable.\n"
     "# - The list of libraries cached in /etc/ld.so.cache\n"
     "# - /lib, followed by /usr/lib\n"
+    "#\n"
+    "\n"
+    "# Check whether a plugin is available for your operating system.\n"
+    "# Each plugin should have a README file in it's lib subfolder.\n"
     "\n");
 
   while (plugins) {
     abuf_appendf(out, "LoadPlugin \"%s\" {\n", plugins->name);
     pl_param = plugins->params;
     while (pl_param) {
-      abuf_appendf(out, "    PlParam \"%s\"\t\"%s\"\n", pl_param->key, pl_param->value);
+      abuf_appendf(out, "    PlParam \"%s\" \"%s\"\n", pl_param->key, pl_param->value);
       pl_param = pl_param->next;
     }
     abuf_puts(out, "}\n"
@@ -993,19 +1142,21 @@ void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf) {
     "# interfaces.\n"
     "\n"
     "InterfaceDefaults\n");
-  olsrd_write_if_autobuf(out, cnf->interface_defaults, false);
+  olsrd_write_if_autobuf(out, cnf, cnf->interface_defaults, false);
   abuf_puts(out,
     "\n"
     "######################################\n"
     "### OLSRd Interfaces configuration ###\n"
     "######################################\n"
-    "# multiple interfaces can be specified for a single configuration block\n"
-    "# multiple configuration blocks can be specified\n"
+    "# Multiple interfaces can be specified for a single configuration block, format:\n"
+    "#   Interface \"<OLSRd-Interface1>\" \"<OLSRd-Interface2>\"\n"
+    "#\n"
+    "# Multiple configuration blocks can be specified.\n"
     "\n");
   first = true;
   while (interf) {
     abuf_appendf(out, "Interface \"%s\"\n", interf->name);
-    olsrd_write_if_autobuf(out, interf->cnf, first);
+    olsrd_write_if_autobuf(out, cnf, interf->cnf, first);
 
     first = false;
     interf = interf->next;
