@@ -443,7 +443,8 @@ olsr_input_hna(union olsr_message *m, struct interface_olsr *in_if __attribute__
 
 #ifdef __linux__
     if (olsr_cnf->smart_gw_active && olsr_is_smart_gateway(&prefix, &mask)) {
-      olsr_update_gateway_entry(&originator, &mask, prefix.prefix_len, msg_seq_number);
+      olsr_update_gateway_entry(&originator, &mask, prefix.prefix_len, msg_seq_number, vtime);
+      continue;
     }
 #endif /* __linux__ */
 
