@@ -39,6 +39,8 @@
  *
  */
 
+#ifdef __linux__
+
 #include "kernel_routes.h"
 #include "ipc_frontend.h"
 #include "log.h"
@@ -75,7 +77,7 @@ static void rtnetlink_read(int sock, void *, unsigned int);
 struct olsr_rtreq {
   struct nlmsghdr n;
   struct rtmsg r;
-  char buf[512];
+  char buf[1024];
 };
 
 struct olsr_ipadd_req {
@@ -96,11 +98,12 @@ int rtnetlink_register_socket(int rtnl_mgrp)
 
   memset(&addr, 0, sizeof(addr));
   addr.nl_family = AF_NETLINK;
-  addr.nl_pid = 0; //kernel will assign appropiate number instead of pid (which is already used by primaray rtnetlink socket to add/delete routes)
+  addr.nl_pid = 0; //kernel will assign appropriate number instead of pid (which is already used by primary rtnetlink socket to add/delete routes)
   addr.nl_groups = rtnl_mgrp;
 
   if (bind(sock,(struct sockaddr *)&addr,sizeof(addr))<0) {
     OLSR_PRINTF(1,"could not bind rtnetlink socket! %s (%d)",strerror(errno), errno);
+    close (sock);
     return -1;
   }
 
@@ -111,14 +114,14 @@ int rtnetlink_register_socket(int rtnl_mgrp)
 static void netlink_process_link(struct nlmsghdr *h)
 {
   struct ifinfomsg *ifi = (struct ifinfomsg *) NLMSG_DATA(h);
-  struct interface *iface;
+  struct interface_olsr *iface;
   struct olsr_if *oif;
   char namebuffer[IF_NAMESIZE];
 
   iface = if_ifwithindex(ifi->ifi_index);
   oif = NULL;
 
-  if (iface == NULL && (ifi->ifi_flags & (IFF_UP|IFF_RUNNING)) == (IFF_UP|IFF_RUNNING)) {
+  if (iface == NULL && (ifi->ifi_flags & IFF_UP) == IFF_UP) {
     if (if_indextoname(ifi->ifi_index, namebuffer)) {
       if ((oif = olsrif_ifwithname(namebuffer)) != NULL) {
         /* try to take interface up, will trigger ifchange */
@@ -133,10 +136,10 @@ static void netlink_process_link(struct nlmsghdr *h)
 
   if (iface == NULL && oif == NULL) {
     /* this is not an OLSR interface */
-    if ((ifi->ifi_flags & IFF_UP) != 0 && (ifi->ifi_flags & IFF_RUNNING) != 0) {
+    if ((ifi->ifi_flags & IFF_UP) != 0) {
       olsr_trigger_ifchange(ifi->ifi_index, NULL, IFCHG_IF_ADD);
     }
-    else if ((ifi->ifi_flags & IFF_UP) == 0 && (ifi->ifi_flags & IFF_RUNNING) == 0){
+    else if ((ifi->ifi_flags & IFF_UP) == 0){
       olsr_trigger_ifchange(ifi->ifi_index, NULL, IFCHG_IF_REMOVE);
     }
   }
@@ -335,9 +338,9 @@ int olsr_os_ifip(int ifindex, union olsr_ip_addr *ip, bool create) {
   return olsr_add_ip(ifindex, ip, NULL, create);
 }
 
-static int olsr_new_netlink_route(int family, int rttable, int if_index, int metric, int protocol,
+int olsr_new_netlink_route(unsigned char family, uint32_t rttable, unsigned int flags, unsigned char scope, int if_index, int metric, int protocol,
     const union olsr_ip_addr *src, const union olsr_ip_addr *gw, const struct olsr_ip_prefix *dst,
-    bool set, bool del_similar) {
+    bool set, bool del_similar, bool blackhole) {
 
   struct olsr_rtreq req;
   int family_size;
@@ -355,9 +358,18 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
 
   memset(&req, 0, sizeof(req));
 
-  req.r.rtm_flags = RTNH_F_ONLINK;
+  req.r.rtm_flags = flags;
   req.r.rtm_family = family;
+#ifndef __ANDROID__
+  if (rttable < 256)
+    req.r.rtm_table = rttable;
+  else {
+    req.r.rtm_table = RT_TABLE_UNSPEC;
+    olsr_netlink_addreq(&req.n, sizeof(req), RTA_TABLE, &rttable, sizeof(rttable));
+  }
+#else
   req.r.rtm_table = rttable;
+#endif
 
   req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
   req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK;
@@ -371,7 +383,11 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
 
   /* RTN_UNSPEC would be the wildcard, but blackhole broadcast or nat roules should usually not conflict */
   /* -> olsr only adds deletes unicast routes */
-  req.r.rtm_type = RTN_UNICAST;
+  if (blackhole) {
+    req.r.rtm_type = RTN_BLACKHOLE;
+  } else {
+    req.r.rtm_type = RTN_UNICAST;
+  }
 
   req.r.rtm_dst_len = dst->prefix_len;
 
@@ -387,10 +403,10 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
   }
   else {
     /* for all our routes */
-    req.r.rtm_scope = RT_SCOPE_UNIVERSE;
+    req.r.rtm_scope = scope;
   }
 
-  if (set || !del_similar) {
+  if ((set || !del_similar) && !blackhole) {
     /* add interface*/
     olsr_netlink_addreq(&req.n, sizeof(req), RTA_OIF, &if_index, sizeof(if_index));
   }
@@ -400,7 +416,7 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
     olsr_netlink_addreq(&req.n, sizeof(req), RTA_PREFSRC, src, family_size);
   }
 
-  if (metric != -1) {
+  if (metric >= 0) {
     /* add metric */
     olsr_netlink_addreq(&req.n, sizeof(req), RTA_PRIORITY, &metric, sizeof(metric));
   }
@@ -446,8 +462,8 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
 void olsr_os_niit_6to4_route(const struct olsr_ip_prefix *dst_v6, bool set) {
   if (olsr_new_netlink_route(AF_INET6,
       ip_prefix_is_mappedv4_inetgw(dst_v6) ? olsr_cnf->rt_table_default : olsr_cnf->rt_table,
-      olsr_cnf->niit6to4_if_index,
-      RT_METRIC_DEFAULT, olsr_cnf->rt_proto, NULL, NULL, dst_v6, set, false)) {
+      RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, olsr_cnf->niit6to4_if_index,
+      olsr_cnf->fib_metric_default, olsr_cnf->rt_proto, NULL, NULL, dst_v6, set, false, false)) {
     olsr_syslog(OLSR_LOG_ERR, ". error while %s static niit route to %s",
         set ? "setting" : "removing", olsr_ip_prefix_to_string(dst_v6));
   }
@@ -456,29 +472,30 @@ void olsr_os_niit_6to4_route(const struct olsr_ip_prefix *dst_v6, bool set) {
 void olsr_os_niit_4to6_route(const struct olsr_ip_prefix *dst_v4, bool set) {
   if (olsr_new_netlink_route(AF_INET,
       ip_prefix_is_v4_inetgw(dst_v4) ? olsr_cnf->rt_table_default : olsr_cnf->rt_table,
-      olsr_cnf->niit4to6_if_index,
-      RT_METRIC_DEFAULT, olsr_cnf->rt_proto, NULL, NULL, dst_v4, set, false)) {
+      RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, olsr_cnf->niit4to6_if_index,
+      olsr_cnf->fib_metric_default, olsr_cnf->rt_proto, NULL, NULL, dst_v4, set, false, false)) {
     olsr_syslog(OLSR_LOG_ERR, ". error while %s niit route to %s",
         set ? "setting" : "removing", olsr_ip_prefix_to_string(dst_v4));
   }
 }
 
-void olsr_os_inetgw_tunnel_route(uint32_t if_idx, bool ipv4, bool set) {
+void olsr_os_inetgw_tunnel_route(uint32_t if_idx, bool ipv4, bool set, uint8_t table) {
   const struct olsr_ip_prefix *dst;
 
   assert(olsr_cnf->ip_version == AF_INET6 || ipv4);
 
   dst = ipv4 ? &ipv4_internet_route : &ipv6_internet_route;
 
-  if (olsr_new_netlink_route(ipv4 ? AF_INET : AF_INET6, olsr_cnf->rt_table_tunnel,
-      if_idx, RT_METRIC_DEFAULT, olsr_cnf->rt_proto, NULL, NULL, dst, set, false)) {
+  if (olsr_new_netlink_route(ipv4 ? AF_INET : AF_INET6, table, RTNH_F_ONLINK, RT_SCOPE_UNIVERSE,
+      if_idx, olsr_cnf->fib_metric_default, olsr_cnf->rt_proto, NULL, NULL, dst, set, false, false)) {
     olsr_syslog(OLSR_LOG_ERR, ". error while %s inetgw tunnel route to %s for if %d",
         set ? "setting" : "removing", olsr_ip_prefix_to_string(dst), if_idx);
   }
 }
 
-static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bool set) {
-  int metric, table;
+static int olsr_os_process_rt_entry(unsigned char af_family, const struct rt_entry *rt, bool set) {
+  int metric;
+  uint32_t table;
   const struct rt_nexthop *nexthop;
   union olsr_ip_addr *src;
   bool hostRoute;
@@ -486,7 +503,7 @@ static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bo
 
   /* calculate metric */
   if (FIBM_FLAT == olsr_cnf->fib_metric) {
-    metric = RT_METRIC_DEFAULT;
+    metric = olsr_cnf->fib_metric_default;
   }
   else {
     metric = set ? rt->rt_best->rtp_metric.hops : rt->rt_metric.hops;
@@ -513,17 +530,6 @@ static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bo
   hostRoute = rt->rt_dst.prefix_len == olsr_cnf->ipsize * 8
       && ipequal(&nexthop->gateway, &rt->rt_dst.prefix);
 
-#if 0
-  {
-    struct ipaddr_str buf1, buf2;
-    olsr_syslog(OLSR_LOG_INFO, "hostroute (%s) = %d == %d && %s == %s",
-        hostRoute ? "true" : "false",
-        rt->rt_dst.prefix_len, (int)(olsr_cnf->ipsize * 8),
-        olsr_ip_to_string(&buf1, &nexthop->gateway),
-        olsr_ip_to_string(&buf2, &rt->rt_dst.prefix));
-  }
-#endif
-
   /* get src ip */
   if (olsr_cnf->use_src_ip_routes) {
     src = &olsr_cnf->unicast_src_ip;
@@ -533,8 +539,8 @@ static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bo
   }
 
   /* create route */
-  err = olsr_new_netlink_route(af_family, table, nexthop->iif_index, metric, olsr_cnf->rt_proto,
-      src, hostRoute ? NULL : &nexthop->gateway, &rt->rt_dst, set, false);
+  err = olsr_new_netlink_route(af_family, table, RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, nexthop->iif_index, metric, olsr_cnf->rt_proto,
+      src, hostRoute ? NULL : &nexthop->gateway, &rt->rt_dst, set, false, false);
 
   /* resolve "File exist" (17) propblems (on orig and autogen routes)*/
   if (set && err == 17) {
@@ -542,12 +548,12 @@ static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bo
     olsr_syslog(OLSR_LOG_ERR, ". auto-deleting similar routes to resolve 'File exists' (17) while adding route!");
 
     /* erase similar rule */
-    err = olsr_new_netlink_route(af_family, table, 0, 0, -1, NULL, NULL, &rt->rt_dst, false, true);
+    err = olsr_new_netlink_route(af_family, table, RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, 0, 0, -1, NULL, NULL, &rt->rt_dst, false, true, false);
 
     if (!err) {
       /* create this rule a second time if delete worked*/
-      err = olsr_new_netlink_route(af_family, table, nexthop->iif_index, metric, olsr_cnf->rt_proto,
-          src, hostRoute ? NULL : &nexthop->gateway, &rt->rt_dst, set, false);
+      err = olsr_new_netlink_route(af_family, table, RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, nexthop->iif_index, metric, olsr_cnf->rt_proto,
+          src, hostRoute ? NULL : &nexthop->gateway, &rt->rt_dst, set, false, false);
     }
     olsr_syslog(OLSR_LOG_ERR, ". %s (%d)", err == 0 ? "successful" : "failed", err);
   }
@@ -583,12 +589,12 @@ static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bo
     hostPrefix.prefix = nexthop->gateway;
     hostPrefix.prefix_len = olsr_cnf->ipsize * 8;
 
-    err = olsr_new_netlink_route(af_family, olsr_cnf->rt_table, nexthop->iif_index,
-        metric, olsr_cnf->rt_proto, src, NULL, &hostPrefix, true, false);
+    err = olsr_new_netlink_route(af_family, olsr_cnf->rt_table, RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, nexthop->iif_index,
+        metric, olsr_cnf->rt_proto, src, NULL, &hostPrefix, true, false, false);
     if (err == 0) {
       /* create this rule a second time if hostrule generation was successful */
-      err = olsr_new_netlink_route(af_family, table, nexthop->iif_index, metric, olsr_cnf->rt_proto,
-          src, hostRoute ? NULL : &nexthop->gateway, &rt->rt_dst, set, false);
+      err = olsr_new_netlink_route(af_family, table, RTNH_F_ONLINK, RT_SCOPE_UNIVERSE, nexthop->iif_index, metric, olsr_cnf->rt_proto,
+          src, &nexthop->gateway, &rt->rt_dst, set, false, false);
     }
     olsr_syslog(OLSR_LOG_ERR, ". %s (%d)", err == 0 ? "successful" : "failed", err);
   }
@@ -599,7 +605,7 @@ static int olsr_os_process_rt_entry(int af_family, const struct rt_entry *rt, bo
 /**
  * Insert a route in the kernel routing table
  *
- * @param destination the route to add
+ * @param rt the route to add
  *
  * @return negative on error
  */
@@ -613,7 +619,7 @@ olsr_ioctl_add_route(const struct rt_entry *rt)
 /**
  *Insert a route in the kernel routing table
  *
- *@param destination the route to add
+ *@param rt the route to add
  *
  *@return negative on error
  */
@@ -627,7 +633,7 @@ olsr_ioctl_add_route6(const struct rt_entry *rt)
 /**
  *Remove a route from the kernel
  *
- *@param destination the route to remove
+ *@param rt the route to remove
  *
  *@return negative on error
  */
@@ -641,7 +647,7 @@ olsr_ioctl_del_route(const struct rt_entry *rt)
 /**
  *Remove a route from the kernel
  *
- *@param destination the route to remove
+ *@param rt the route to remove
  *
  *@return negative on error
  */
@@ -651,6 +657,7 @@ olsr_ioctl_del_route6(const struct rt_entry *rt)
   OLSR_PRINTF(2, "KERN: Deleting %s\n", olsr_rt_to_string(rt));
   return olsr_os_process_rt_entry(AF_INET6, rt, false);
 }
+#endif /* __linux__ */
 
 /*
  * Local Variables:
